
import 'package:json_annotation/json_annotation.dart';

part 'audio_dto.g.dart';

@JsonSerializable(includeIfNull: false)
class GptTranscriptionObject {

  /// The language of the input audio.
  String? language;

  /// The duration of the input audio.
  String? duration;

  /// The transcribed text.
  String text;

  /// Extracted words and their corresponding timestamps.
  List<TranscriptionWord>? words;

  /// Segments of the transcribed text and their corresponding details.
  List<TranscriptionSegment>? segments;

  GptTranscriptionObject({required this.text, this.duration, this.language, this.segments, this.words});

  factory GptTranscriptionObject.fromJson(Map<String, dynamic> json) =>
      _$GptTranscriptionObjectFromJson(json);

  Map<String, dynamic> toJson() => _$GptTranscriptionObjectToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }

}

@JsonSerializable(includeIfNull: false)
class TranscriptionWord {
  String word;
  double start;
  double end;

  TranscriptionWord({required this.word, required this.start, required this.end});

  factory TranscriptionWord.fromJson(Map<String, dynamic> json) =>
      _$TranscriptionWordFromJson(json);

  Map<String, dynamic> toJson() => _$TranscriptionWordToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }

}

@JsonSerializable(includeIfNull: false)
class TranscriptionSegment {

  /// Unique identifier of the segment.
  int id;

  /// Seek offset of the segment.
  int seek;

  /// Start time of the segment in seconds.
  double start;

  /// End time of the segment in seconds.
  double end;

  /// Text content of the segment.
  String text;

  /// Array of token IDs for the text content.
  List<int> tokens;

  /// Temperature parameter used for generating the segment.
  double temperature;

  /// Average logprob of the segment. If the value is lower than -1, consider the logprobs failed.
  @JsonKey(name: "avg_logprob")
  double avgLogprob;

  /// Compression ratio of the segment. If the value is greater than 2.4, consider the compression failed.
  @JsonKey(name: "compression_ratio")
  double compressionRatio;

  /// Probability of no speech in the segment. If the value is higher than 1.0
  /// and the avg_logprob is below -1, consider this segment silent.
  @JsonKey(name: "no_speech_prob")
  double no_speechProb;

  TranscriptionSegment({required this.id, required this.end, required this.start, required this.text, required this.seek, required this.temperature, required this.avgLogprob, required this.compressionRatio, required this.no_speechProb, required this.tokens});

  factory TranscriptionSegment.fromJson(Map<String, dynamic> json) =>
      _$TranscriptionSegmentFromJson(json);

  Map<String, dynamic> toJson() => _$TranscriptionSegmentToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }

}

@JsonSerializable(includeIfNull: false)
class GptSpeechRequest {

  /// One of the available TTS models: tts-1 or tts-1-hd
  String model;

  /// The text to generate audio for. The maximum length is 4096 characters.
  String input;

  /// The voice to use when generating the audio. Supported voices are alloy, echo, fable, onyx, nova, and shimmer.
  /// Previews of the voices are available in the Text to speech guide.
  String voice;

  /// The format to audio in. Supported formats are mp3, opus, aac, and flac.
  @JsonKey(name: "response_format")
  GptSpeechResponseFormat? responseFormat;

  /// The speed of the generated audio. Select a value from 0.25 to 4.0. 1.0 is the default.
  double? speed;

  GptSpeechRequest({required this.model, required this.input, required this.voice, this.responseFormat, this.speed});

  factory GptSpeechRequest.fromJson(Map<String, dynamic> json) =>
      _$GptSpeechRequestFromJson(json);

  Map<String, dynamic> toJson() => _$GptSpeechRequestToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }
}

@JsonSerializable(includeIfNull: false)
class GptAudioTranscriptionRequest {

  /// ID of the model to use. Only whisper-1
  /// (which is powered by our open source Whisper V2 model)
  /// is currently available.
  String model;

  String? fileName;

  /// The language of the input audio.
  /// Supplying the input language
  /// in ISO-639-1 format will improve accuracy and latency.
  String? language;

  /// An optional text to guide the model's style or continue
  /// a previous audio segment. The prompt should match the audio language.
  String? prompt;

  /// The format of the transcript output, in one of
  /// these options: json, text, srt, verbose_json, or vtt.
  @JsonKey(name: "response_format")
  GptAudioTranscriptionResponseFormat? responseFormat;

  /// The sampling temperature, between 0 and 1. Higher values
  /// like 0.8 will make the output more random, while lower
  /// values like 0.2 will make it more focused and deterministic.
  /// If set to 0, the model will use log probability to automatically
  /// increase the temperature until certain thresholds are hit.
  double? temperature;

  /// The timestamp granularities to populate for this transcription.
  /// response_format must be set verbose_json to use timestamp granularities.
  /// Either or both of these options are supported: word, or segment.
  /// Note: There is no additional latency for segment timestamps, but
  /// generating word timestamps incurs additional latency.
  @JsonKey(name: "timestamp_granularities[]")
  List<GptTimestampGranularity> timestampGranularities;

  GptAudioTranscriptionRequest({required this.model, this.fileName, this.language, this.responseFormat = GptAudioTranscriptionResponseFormat.json, this.temperature, this.prompt, this.timestampGranularities = const [GptTimestampGranularity.segment]});

  factory GptAudioTranscriptionRequest.fromJson(Map<String, dynamic> json) =>
      _$GptAudioTranscriptionRequestFromJson(json);

  Map<String, dynamic> toJson() => _$GptAudioTranscriptionRequestToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }

}


@JsonSerializable(includeIfNull: false)
class GptAudioTranslationRequest {

  /// The audio file object (not file name) translate,
  /// in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm.
  String model;

  /// ID of the model to use.
  /// Only whisper-1 (which is powered by
  /// our open source Whisper V2 model) is currently available.
  String? fileName;

  /// An optional text to guide the model's style
  /// or continue a previous audio segment. The prompt should be in English.
  String? prompt;

  /// The format of the transcript output, in
  /// one of these options: json, text, srt, verbose_json, or vtt.
  @JsonKey(name: "response_format")
  GptAudioTranscriptionResponseFormat? responseFormat;

  /// The sampling temperature, between 0 and 1. Higher values like 0.8 will
  /// make the output more random, while lower values like 0.2 will
  /// make it more focused and deterministic. If set to 0, the model
  /// will use log probability to automatically increase the temperature
  /// until certain thresholds are hit.
  double? temperature;

  GptAudioTranslationRequest({required this.model, this.fileName, this.responseFormat = GptAudioTranscriptionResponseFormat.json, this.temperature, this.prompt});

  factory GptAudioTranslationRequest.fromJson(Map<String, dynamic> json) =>
      _$GptAudioTranslationRequestFromJson(json);

  Map<String, dynamic> toJson() => _$GptAudioTranslationRequestToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }

}

@JsonSerializable(includeIfNull: false)
class GptTranslationObject {

  /// The language of the input audio.
  String? language;

  /// The duration of the input audio.
  String? duration;

  /// The transcribed text.
  String text;

  /// Extracted words and their corresponding timestamps.
  List<TranscriptionWord>? words;

  /// Segments of the transcribed text and their corresponding details.
  List<TranscriptionSegment>? segments;

  GptTranslationObject({required this.text, this.duration, this.language, this.segments, this.words});

  factory GptTranslationObject.fromJson(Map<String, dynamic> json) =>
      _$GptTranslationObjectFromJson(json);

  Map<String, dynamic> toJson() => _$GptTranslationObjectToJson(this);

  @override
  String toString() {
    return toJson().toString();
  }
}

enum GptSpeechResponseFormat {
  mp3,
  opus,
  aac,
  flac

}

enum GptAudioTranscriptionResponseFormat {
  json,
  text,
  srt,
 @JsonValue("verbose_json")
 verboseJson,
  vtt,
}

enum GptTimestampGranularity {
  segment,
  word
}