import 'dart:convert';
import 'dart:typed_data';
import 'package:another_chatgpt/error_models.dart';
import 'package:http/http.dart' as http;

import 'audio_dto.dart';

class AudioService {
  final String baseUrl;
  final bool secure;

  AudioService({required this.baseUrl, required this.secure});

  ///
  /// Generates audio from the input text.
  ///
  Future<Uint8List> createSpeech({
    required String apiKey,
    required String organizationId,
    required GptSpeechRequest request,
  }) async {
    Map<String, String> headers = {
      "Authorization": "Bearer $apiKey",
      "OpenAI-Organization": organizationId,
      'Content-Type': 'application/json',
      "Access-Control-Allow-Origin": "*", // Required for CORS support to work
    };

    Uri endpoint;
    if (secure) {
      endpoint = Uri.https(baseUrl, "/v1/audio/speech");
    } else {
      endpoint = Uri.http(baseUrl, "/v1/audio/speech");
    }

    var response = await http.post(endpoint,
        headers: headers, body: jsonEncode(request.toJson()));

    if (response.statusCode == 200 || response.statusCode == 201) {
      return response.bodyBytes;
    } else {
      var error = ServerError.fromJson(jsonDecode(response.body));
      throw AudioException(
          statusCode: response.statusCode, message: error.message);
    }
  }

  ///
  /// Transcribes audio into the input language.
  ///
  Future<GptTranscriptionObject> transcribeAudio(
      {required String apiKey,
      required String organizationId,
      required GptAudioTranscriptionRequest request,
      required Uint8List fileBytes}) async {
    Map<String, String> headers = {
      "Authorization": "Bearer $apiKey",
      "OpenAI-Organization": organizationId,
      'Content-Type': 'application/json',
      "Access-Control-Allow-Origin": "*", // Required for CORS support to work
    };

    Uri endpoint;
    if (secure) {
      endpoint = Uri.https(baseUrl, "/v1/audio/transcriptions");
    } else {
      endpoint = Uri.http(baseUrl, "/v1/audio/transcriptions");
    }

    String? targetFilename = request.fileName;
    request.fileName = null;
    List<GptTimestampGranularity> granularities =
        request.timestampGranularities;

    // TODO convert ganularity to comman separated string.
    String outGranularities = granularities.map((e) => e.name).join(",");

    var httpRequest = http.MultipartRequest('POST', endpoint)
      ..headers.addAll(headers)
      ..fields.addAll(
        request.toJson().map(
              (key, value) {

                if(key == "timestamp_granularities[]") {
                  return MapEntry(key, outGranularities);
                }

                return MapEntry(
                  key,
                  value.toString(),
                );
              }
            ),
      )
      ..files.add(http.MultipartFile.fromBytes("file", fileBytes,
          filename: targetFilename));

    var response = await httpRequest.send();

    if (response.statusCode == 200 || response.statusCode == 201) {
      return GptTranscriptionObject.fromJson(jsonDecode(
          const Utf8Decoder().convert(await response.stream.toBytes())));
    } else {
      var error = ServerError.fromJson(jsonDecode(
          const Utf8Decoder().convert(await response.stream.toBytes())));
      throw AudioException(
          statusCode: response.statusCode, message: error.message);
    }
  }

  /// Translates audio into English.
  Future<GptTranslationObject> translateAudio(
      {required String apiKey,
      required String organizationId,
      required GptAudioTranslationRequest request,
      required Uint8List fileBytes}) async {
    Map<String, String> headers = {
      "Authorization": "Bearer $apiKey",
      "OpenAI-Organization": organizationId,
      'Content-Type': 'application/json',
      "Access-Control-Allow-Origin": "*", // Required for CORS support to work
    };

    Uri endpoint;
    if (secure) {
      endpoint = Uri.https(baseUrl, "/v1/audio/translations");
    } else {
      endpoint = Uri.http(baseUrl, "/v1/audio/translations");
    }

    String? targetFilename = request.fileName;
    request.fileName = null;
    var httpRequest = http.MultipartRequest('POST', endpoint)
      ..headers.addAll(headers)
      ..fields.addAll(
          request.toJson().map((key, value) => MapEntry(key, value.toString())))
      ..files.add(http.MultipartFile.fromBytes("file", fileBytes,
          filename: targetFilename));

    var response = await httpRequest.send();

    if (response.statusCode == 200 || response.statusCode == 201) {
      return GptTranslationObject.fromJson(jsonDecode(
          const Utf8Decoder().convert(await response.stream.toBytes())));
    } else {
      var error = ServerError.fromJson(jsonDecode(
          const Utf8Decoder().convert(await response.stream.toBytes())));
      throw AudioException(
          statusCode: response.statusCode, message: error.message);
    }
  }
}

///
/// Exception related to the Audio Service
///
class AudioException implements Exception {
  final String? message;
  final int statusCode;

  AudioException({required this.statusCode, this.message});

  @override
  String toString() {
    return "AudioException: $statusCode - $message";
  }
}
